/*
    Copyright (C) 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013
    Ben Kibbey <bjk@luxsci.net>

    This file is part of pwmd.

    Pwmd is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    Pwmd is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Pwmd.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef XML_H
#define XML_H

#include <libxml/tree.h>
#include <libxml/parser.h>
#include <libxml/xpath.h>

#include "util-string.h"

struct element_list_s
{
  struct slist_s *list;
  char **elements;
  char *prefix;
  int resolving;
  int recurse;
  int verbose;
  int with_target;
  char **req;
};

gpg_error_t new_root_element (xmlDocPtr doc, char *name);
xmlDocPtr new_document (void);
gpg_error_t list_root_elements (xmlDocPtr doc, struct string_s **, int, int);
int valid_xml_element (xmlChar * element);
xmlNodePtr find_elements (xmlDocPtr doc, xmlNodePtr node,
			  char **req, gpg_error_t * error, int *target,
			  xmlNodePtr (*found_fn) (xmlNodePtr, char **,
						  gpg_error_t *, char **,
						  void *),
			  xmlNodePtr (*not_found_fn) (xmlNodePtr, char **,
						      gpg_error_t *, void *),
			  int is_list_command, int recursion_depth,
			  void *data, int stop);
xmlNodePtr find_root_element (xmlDocPtr doc, char ***req, gpg_error_t * error,
			      int *target, int recursion_depth, int stop);
xmlNodePtr create_elements_cb (xmlNodePtr node, char **elements,
			       gpg_error_t * error, void *data);
xmlNodePtr create_target_elements_cb (xmlNodePtr node, char **elements,
				      gpg_error_t * error, void *data);
int is_literal_element (char **element);
int valid_element_path (char **path, int content);
xmlNodePtr find_text_node (xmlNodePtr node);
gpg_error_t create_path_list (xmlDocPtr doc, struct element_list_s *elements,
			      char *path);
gpg_error_t recurse_xpath_nodeset (xmlDocPtr doc, xmlNodeSetPtr nodes,
				   xmlChar * value, xmlBufferPtr * result,
				   int, const xmlChar *);
gpg_error_t add_attribute (xmlNodePtr node, const char *name,
			   const char *value);
xmlChar *node_has_attribute (xmlNodePtr n, xmlChar * attr);
gpg_error_t delete_attribute (xmlNodePtr n, const xmlChar * name);
gpg_error_t convert_pre_212_elements (xmlDocPtr doc);
gpg_error_t validate_import (xmlNodePtr);
xmlNodePtr find_element (xmlNodePtr node, char *element, xmlNodePtr stop);
gpg_error_t update_element_mtime (xmlNodePtr n);
gpg_error_t unlink_node (xmlNodePtr n);
gpg_error_t parse_doc (const char *xml, size_t len, xmlDocPtr *doc);
gpg_error_t attr_ctime (xmlNodePtr n);
gpg_error_t build_realpath (xmlDocPtr doc, char *line,
			    struct string_s **result);
gpg_error_t validate_target_attribute (xmlDocPtr doc, const char *src,
				       xmlNodePtr dst);
int valid_xml_attribute (const char *str);

#endif
